@@ -44,11 +44,11 @@ def render(recipe_path, config=None, variants=None, permit_unsatisfiable_variant
                                     permit_unsatisfiable_variants=permit_unsatisfiable_variants)
     output_metas = OrderedDict()
     for meta, download, render_in_env in metadata_tuples:
-        if not meta.skip():
+        if not meta.skip() or not config.trim_skip:
             for od, om in meta.get_output_metadata_set(
                     permit_unsatisfiable_variants=permit_unsatisfiable_variants,
                     permit_undefined_jinja=not finalize):
-                if not om.skip():
+                if not om.skip() or not config.trim_skip:
                     # only show conda packages right now
                     if 'type' not in od or od['type'] == 'conda':
                         if finalize and not om.final:
