@@ -7,6 +7,7 @@
 from __future__ import absolute_import, division, print_function
 
 import argparse
+from glob2 import glob
 import logging
 import os
 import sys
@@ -16,12 +17,13 @@ import filelock
 import conda_build.api as api
 import conda_build.build as build
 import conda_build.utils as utils
-from conda_build.cli.main_render import (set_language_env_vars, RecipeCompleter,
-                                         get_render_parser, bldpkg_path)
-from conda_build.conda_interface import add_parser_channels, url_path, binstar_upload
+from conda_build.conda_interface import (add_parser_channels, url_path, binstar_upload,
+                                         cc_conda_build)
+from conda_build.cli.main_render import get_render_parser
 import conda_build.source as source
-from conda_build.utils import print_skip_message, LoggingContext
+from conda_build.utils import LoggingContext
 from conda_build.config import Config
+from os.path import abspath, expanduser, expandvars
 
 on_win = (sys.platform == 'win32')
 
@@ -60,7 +62,7 @@ different sets of packages."""
         action="store_false",
         help="Don't include the recipe inside the built package.",
         dest='include_recipe',
-        default=True,
+        default=cc_conda_build.get('include_recipe', 'true').lower() == 'true',
     )
     p.add_argument(
         '-s', "--source",
@@ -95,15 +97,15 @@ different sets of packages."""
         'recipe',
         metavar='RECIPE_PATH',
         nargs='+',
-        choices=RecipeCompleter(),
         help="Path to recipe directory.  Pass 'purge' here to clean the "
         "work and test intermediates.",
     )
     p.add_argument(
         '--skip-existing',
         action='store_true',
-        help="""Skip recipes for which there already exists an existing build
-        (locally or in the channels). """
+        help=("Skip recipes for which there already exists an existing build"
+              "(locally or in the channels)."),
+        default=cc_conda_build.get('skip_existing', 'false').lower() == 'true',
     )
     p.add_argument(
         '--keep-old-work',
@@ -122,6 +124,7 @@ different sets of packages."""
         '-q', "--quiet",
         action="store_true",
         help="do not display progress bar",
+        default=cc_conda_build.get('quiet', 'false').lower() == 'true',
     )
     p.add_argument(
         '--debug',
@@ -130,11 +133,13 @@ different sets of packages."""
     )
     p.add_argument(
         '--token',
-        help="Token to pass through to anaconda upload"
+        help="Token to pass through to anaconda upload",
+        default=cc_conda_build.get('anaconda_token'),
     )
     p.add_argument(
         '--user',
-        help="User/organization to upload packages to on anaconda.org or pypi"
+        help="User/organization to upload packages to on anaconda.org or pypi",
+        default=cc_conda_build.get('user'),
     )
     p.add_argument(
         '--no-force-upload',
@@ -146,7 +151,7 @@ different sets of packages."""
     pypi_grp = p.add_argument_group("PyPI upload parameters (twine)")
     pypi_grp.add_argument(
         '--password',
-        help="password to use when uploading packages to pypi"
+        help="password to use when uploading packages to pypi",
     )
     pypi_grp.add_argument(
         '--sign', default=False,
@@ -162,17 +167,21 @@ different sets of packages."""
     )
     pypi_grp.add_argument(
         '--config-file',
-        help="path to .pypirc file to use when uploading to pypi"
+        help="path to .pypirc file to use when uploading to pypi",
+        default=(abspath(expanduser(expandvars(cc_conda_build.get('pypirc'))))
+                 if cc_conda_build.get('pypirc')
+                 else cc_conda_build.get('pypirc')),
     )
     pypi_grp.add_argument(
-        '--repository', '-r', default='pypitest',
-        help="PyPI repository to upload to"
+        '--repository', '-r', help="PyPI repository to upload to",
+        default=cc_conda_build.get('pypi_repository', 'pypitest'),
     )
     p.add_argument(
         "--no-activate",
         action="store_false",
         help="do not activate the build and test envs; just prepend to PATH",
         dest='activate',
+        default=cc_conda_build.get('activate', 'true').lower() == 'true',
     )
     p.add_argument(
         "--no-build-id",
@@ -180,6 +189,8 @@ different sets of packages."""
         help=("do not generate unique build folder names.  Use if having issues with "
               "paths being too long."),
         dest='set_build_id',
+        # note: inverted - dest stores positive logic
+        default=cc_conda_build.get('set_build_id', 'true').lower() == 'true',
     )
     p.add_argument(
         "--croot",
@@ -187,9 +198,10 @@ different sets of packages."""
               "to this call of conda-build.")
     )
     p.add_argument(
-        "--no-verify",
+        "--verify",
         action="store_true",
-        help=("do not run verification on recipes or packages when building")
+        help="do not run verification on recipes or packages when building",
+        default=cc_conda_build.get('verify', 'false').lower() == 'true',
     )
     p.add_argument(
         "--output-folder",
@@ -235,18 +247,58 @@ different sets of packages."""
               "tests, but ultimately fail on installed systems.")
     )
     p.add_argument(
-        "--long-test-prefix", default=False, action="store_true",
+        "--error-overlinking", dest='error_overlinking', action="store_true",
+        help=("Enable error when shared libraries from transitive dependencies are directly"
+              "linked to any executables or shared libraries in built packages.  This is disabled"
+              "by default, but will be enabled by default in conda-build 4.0."),
+        default=cc_conda_build.get('error_overlinking', 'false').lower() == 'true',
+    )
+    p.add_argument(
+        "--no-error-overlinking", dest='error_overlinking', action="store_false",
+        help=("Disable error when shared libraries from transitive dependencies are directly"
+              "linked to any executables or shared libraries in built packages.  This is currently"
+              "the default behavior, but will change in conda-build 4.0."),
+        default=cc_conda_build.get('error_overlinking', 'false').lower() == 'true',
+    )
+    p.add_argument(
+        "--long-test-prefix", default=True, action="store_false",
         help=("Use a long prefix for the test prefix, as well as the build prefix.  Affects only "
-              "Linux and Mac.  Prefix length matches the --prefix-length flag.  This is off by "
-              "default in conda-build 2.x, but will be enabled by default in conda-build 3.0.")
+              "Linux and Mac.  Prefix length matches the --prefix-length flag.  This is on by "
+              "default in conda-build 3.0+")
     )
     p.add_argument(
         "--no-long-test-prefix", dest="long_test_prefix", action="store_false",
         help=("Do not use a long prefix for the test prefix, as well as the build prefix."
-              "  Affects only Linux and Mac.  Prefix length matches the --prefix-length flag.  "
-              "This is currently the default behavior, and does not need to be set.  This will"
-              " change in conda-build 3.0 (long-test-prefix enabled by default)")
+              "  Affects only Linux and Mac.  Prefix length matches the --prefix-length flag.  ")
     )
+    p.add_argument(
+        '--keep-going', '-k', action='store_true',
+        help=("When running tests, keep going after each failure.  Default is to stop on the first "
+              "failure.")
+    )
+    p.add_argument(
+        '--cache-dir',
+        help=('Path to store the source files (archives, git clones, etc.) during the build.'),
+        default=(abspath(expanduser(expandvars(cc_conda_build.get('cache_dir'))))
+                 if cc_conda_build.get('cache_dir')
+                 else cc_conda_build.get('cache_dir')),
+    )
+    p.add_argument(
+        "--no-copy-test-source-files", dest="copy_test_source_files", action="store_false",
+        default=cc_conda_build.get('copy_test_source_files', 'true').lower() == 'true',
+        help=("Disables copying the files necessary for testing the package into "
+              "the info/test folder.  Passing this argument means it may not be possible "
+              "to test the package without internet access.  There is also a danger that "
+              "the source archive(s) containing the files could become unavailable sometime "
+              "in the future.")
+    )
+    p.add_argument(
+        '--merge-build-host', action="store_true",
+        help=('Merge the build and host directories, even when host section or compiler '
+              'jinja2 is present'),
+        default=cc_conda_build.get('merge_build_host', 'false').lower() == 'true',
+    )
+
     add_parser_channels(p)
 
     args = p.parse_args(args)
@@ -255,17 +307,16 @@ different sets of packages."""
 
 def output_action(recipe, config):
     with LoggingContext(logging.CRITICAL + 1):
-        metadata, _, _ = api.render(recipe, config=config)
-        if metadata.skip():
-            print_skip_message(metadata)
-        else:
-            print(bldpkg_path(metadata))
+        config.verbose = False
+        config.debug = False
+        paths = api.get_output_file_paths(recipe, config=config)
+        print('\n'.join(sorted(paths)))
 
 
 def source_action(recipe, config):
-    metadata, _, _ = api.render(recipe, config=config)
-    source.provide(metadata, config=config)
-    print('Source tree in:', config.work_dir)
+    metadata = api.render(recipe, config=config)[0][0]
+    source.provide(metadata)
+    print('Source tree in:', metadata.config.work_dir)
 
 
 def test_action(recipe, config):
@@ -282,7 +333,7 @@ def execute(args):
     build.check_external()
 
     # change globals in build module, see comment there as well
-    channel_urls = args.channel or ()
+    channel_urls = args.__dict__.get('channel') or args.__dict__.get('channels') or ()
     config.channel_urls = []
 
     for url in channel_urls:
@@ -307,14 +358,13 @@ def execute(args):
         config.clean_pkgs()
         return
 
-    set_language_env_vars(args, parser, config=config, execute=execute)
-
     action = None
+    outputs = None
     if args.output:
         action = output_action
-        logging.basicConfig(level=logging.ERROR)
         config.verbose = False
         config.quiet = True
+        config.debug = False
     elif args.test:
         action = test_action
     elif args.source:
@@ -322,15 +372,36 @@ def execute(args):
     elif args.check:
         action = check_action
 
-    if action:
-        for recipe in args.recipe:
-            action(recipe, config)
+    if action == test_action:
+        failed_recipes = []
+        recipes = [item for sublist in
+                   [glob(os.path.abspath(recipe)) if '*' in recipe
+                                                  else [recipe] for recipe in args.recipe]
+                   for item in sublist]
+        for recipe in recipes:
+            try:
+                action(recipe, config)
+            except:
+                if not args.keep_going:
+                    raise
+                else:
+                    failed_recipes.append(recipe)
+                    continue
+        if failed_recipes:
+            print("Failed recipes:")
+            for recipe in failed_recipes:
+                print("  - %s" % recipe)
+            sys.exit(len(failed_recipes))
+        else:
+            print("All tests passed")
         outputs = []
 
+    elif action:
+        outputs = [action(recipe, config) for recipe in args.recipe]
     else:
         outputs = api.build(args.recipe, post=args.post, build_only=args.build_only,
                             notest=args.notest, already_built=None, config=config,
-                            noverify=args.no_verify)
+                            verify=args.verify)
 
     if not args.output and len(utils.get_build_folders(config.croot)) > 0:
         build.print_build_intermediate_warning(config)
