@@ -28,8 +28,10 @@ except ImportError:
     # no need to patch if it doesn't exist
     pass
 
+conda_43 = parse_version(CONDA_VERSION) >= parse_version("4.3")
+conda_44 = parse_version(CONDA_VERSION) >= parse_version("4.4")
 
-if parse_version(CONDA_VERSION) >= parse_version("4.4"):
+if conda_44:
     from conda.exports import display_actions, execute_actions, execute_plan, install_actions
 else:
     from conda.plan import display_actions, execute_actions, execute_plan, install_actions
@@ -37,12 +39,17 @@ else:
 display_actions, execute_actions, execute_plan = display_actions, execute_actions, execute_plan
 install_actions = install_actions
 
+try:
+    # Conda 4.4+
+    from conda.exports import _toposort
+except ImportError:
+    from conda.toposort import _toposort
+_toposort = _toposort
 
-if parse_version(CONDA_VERSION) >= parse_version("4.3"):
+if conda_43:
     from conda.exports import TmpDownload, download, handle_proxy_407  # NOQA
     from conda.exports import untracked, walk_prefix  # NOQA
     from conda.exports import MatchSpec, NoPackagesFound, Resolve, Unsatisfiable, normalized_version  # NOQA
-    from conda.exports import KEYS, KEYS_DIR, hash_file  # NOQA
     from conda.exports import human_bytes, hashsum_file, md5_file, memoized, unix_path_to_win, win_path_to_unix, url_path  # NOQA
     from conda.exports import get_index  # NOQA
     from conda.exports import (Completer, InstalledPackages, add_parser_channels,
@@ -57,13 +64,12 @@ if parse_version(CONDA_VERSION) >= parse_version("4.3"):
     from conda.exports import VersionOrder  # NOQA
     from conda.exports import dist_str_in_index
     from conda.core.package_cache import ProgressiveFetchExtract
-    from conda.models.dist import Dist
+    from conda.models.dist import Dist, IndexRecord  # NOQA
 
 else:
     from conda.fetch import TmpDownload, download, handle_proxy_407  # NOQA
     from conda.misc import untracked, walk_prefix  # NOQA
     from conda.resolve import MatchSpec, NoPackagesFound, Resolve, Unsatisfiable, normalized_version  # NOQA
-    from conda.signature import KEYS, KEYS_DIR, hash_file  # NOQA
     from conda.utils import human_bytes, hashsum_file, md5_file, memoized, unix_path_to_win, win_path_to_unix, url_path  # NOQA
     from conda.api import get_index  # NOQA
     from conda.cli.common import (Completer, InstalledPackages, add_parser_channels,
@@ -81,11 +87,11 @@ else:
 
     class ProgressiveFetchExtract(object): pass  # NOQA
     class Dist(object): pass # NOQA
+    class IndexRecord(object): pass # NOQA
 
 TmpDownload = TmpDownload
 download, handle_proxy_407, untracked, walk_prefix = download, handle_proxy_407, untracked, walk_prefix  # NOQA
 MatchSpec, Resolve, normalized_version = MatchSpec, Resolve, normalized_version
-KEYS, KEYS_DIR, hash_file = KEYS, KEYS_DIR, hash_file
 human_bytes, hashsum_file, md5_file, memoized = human_bytes, hashsum_file, md5_file, memoized
 unix_path_to_win, win_path_to_unix, url_path = unix_path_to_win, win_path_to_unix, url_path
 get_index, Completer, InstalledPackages = get_index, Completer, InstalledPackages
@@ -107,7 +113,7 @@ else:
 configparser = configparser
 
 
-if parse_version(CONDA_VERSION) >= parse_version("4.3"):
+if conda_43:
     from conda.exports import FileMode, PathType
     FileMode, PathType = FileMode, PathType
     from conda.exports import EntityEncoder
@@ -266,7 +272,6 @@ else:
 
     del locals()['cc']
 
-
 CondaError, CondaHTTPError, get_prefix, LinkError = CondaError, CondaHTTPError, get_prefix, LinkError  # NOQA
 LockError, non_x86_linux_machines, NoPackagesFoundError = LockError, non_x86_linux_machines, NoPackagesFoundError  # NOQA
 PaddingError, UnsatisfiableError = PaddingError, UnsatisfiableError
@@ -406,6 +411,20 @@ def which_prefix(path):
     return prefix
 
 
+def get_installed_version(prefix, pkgs):
+    """Primarily used by conda-forge, but may be useful in general for checking when a package
+    needs to be updated"""
+    from conda_build.utils import ensure_list
+    pkgs = ensure_list(pkgs)
+    linked_pkgs = linked(prefix)
+    versions = {}
+    for pkg in pkgs:
+        vers_inst = [dist.split('::', 1)[-1].rsplit('-', 2)[1] for dist in linked_pkgs
+            if dist.split('::', 1)[-1].rsplit('-', 2)[0] == pkg]
+        versions[pkg] = vers_inst[0] if len(vers_inst) == 1 else None
+    return versions
+
+
 # when deactivating envs (e.g. switching from root to build/test) this env var is used,
 # except the PR that removed this has been reverted (for now) and Windows doesnt need it.
 env_path_backup_var_exists = os.environ.get('CONDA_PATH_BACKUP', None)
