@@ -4,17 +4,14 @@ Module to handle generating test files.
 
 from __future__ import absolute_import, division, print_function
 
-import glob
-import logging
 import os
-from os.path import join, exists, isdir
+from os.path import join, exists
 import sys
 
-from conda_build.utils import copy_into, get_ext_files, on_win, ensure_list
-from conda_build import source
+from conda_build.utils import copy_into, on_win, ensure_list
 
 
-def create_files(dir_path, m, config):
+def create_files(m, test_dir=None):
     """
     Create the test files for pkg in the directory given.  The resulting
     test files are configuration (i.e. platform, architecture, Python and
@@ -22,38 +19,28 @@ def create_files(dir_path, m, config):
     Return False, if the package has no tests (for any configuration), and
     True if it has.
     """
+    if not test_dir:
+        test_dir = m.config.test_dir
     has_files = False
+    if not os.path.isdir(test_dir):
+        os.makedirs(test_dir)
+
+    recipe_dir = m.path or m.meta.get('extra', {}).get('parent_recipe', {}).get('path')
+
     for fn in ensure_list(m.get_value('test/files', [])):
         has_files = True
-        path = join(m.path, fn)
+        path = join(recipe_dir, fn)
+        if not os.path.isdir(os.path.dirname(path)):
+            os.makedirs(os.path.dirname(path))
         # disable locking to avoid locking a temporary directory (the extracted test folder)
-        copy_into(path, join(dir_path, fn), config.timeout, locking=False)
-    # need to re-download source in order to do tests
-    if m.get_value('test/source_files') and not isdir(config.work_dir):
-        source.provide(m, config=config)
-    for pattern in ensure_list(m.get_value('test/source_files', [])):
-        if on_win and '\\' in pattern:
-            raise RuntimeError("test/source_files paths must use / "
-                                "as the path delimiter on Windows")
-        has_files = True
-        files = glob.glob(join(config.work_dir, pattern))
-        if not files:
-            raise RuntimeError("Did not find any source_files for test with pattern %s", pattern)
-        for f in files:
-            try:
-                # disable locking to avoid locking a temporary directory (the extracted test folder)
-                copy_into(f, f.replace(config.work_dir, config.test_dir), config.timeout,
-                          locking=False)
-            except OSError as e:
-                log = logging.getLogger(__name__)
-                log.warn("Failed to copy {0} into test files.  Error was: {1}".format(f, str(e)))
-        for ext in '.pyc', '.pyo':
-            for f in get_ext_files(config.test_dir, ext):
-                os.remove(f)
+        copy_into(path, join(test_dir, fn), m.config.timeout, locking=False,
+                  clobber=True)
     return has_files
 
 
-def create_shell_files(dir_path, m, config):
+def create_shell_files(m, test_dir=None):
+    if not test_dir:
+        test_dir = m.config.test_dir
     has_tests = False
     ext = '.bat' if sys.platform == 'win32' else '.sh'
     name = 'no-file'
@@ -71,22 +58,28 @@ def create_shell_files(dir_path, m, config):
 
     if exists(join(m.path, name)):
         # disable locking to avoid locking a temporary directory (the extracted test folder)
-        copy_into(join(m.path, name), dir_path, config.timeout, locking=False)
+        copy_into(join(m.path, name), test_dir, m.config.timeout, locking=False)
         has_tests = True
 
-    with open(join(dir_path, name), 'a') as f:
-        f.write('\n\n')
-        for cmd in ensure_list(m.get_value('test/commands', [])):
-            f.write(cmd)
-            f.write('\n')
-            if sys.platform == 'win32':
-                f.write("if errorlevel 1 exit 1\n")
-            has_tests = True
+    commands = ensure_list(m.get_value('test/commands', []))
+    if commands:
+        with open(join(test_dir, name), 'a') as f:
+            f.write('\n\n')
+            if not on_win:
+                f.write('set -ex\n\n')
+            f.write('\n\n')
+            for cmd in commands:
+                f.write(cmd)
+                f.write('\n')
+                if on_win:
+                    f.write("IF %ERRORLEVEL% NEQ 0 exit 1\n")
+                has_tests = True
+            f.write('exit 0\n')
 
-    return has_tests
+    return has_tests or os.path.isfile(os.path.join(m.config.test_dir, name))
 
 
-def _create_test_files(dir_path, m, ext, comment_char='# '):
+def _create_test_files(m, test_dir, ext, comment_char='# '):
     # the way this works is that each output needs to explicitly define a test script to run
     #   They do not automatically pick up run_test.*, but can be pointed at that explicitly.
     name = 'run_test' + ext
@@ -98,7 +91,7 @@ def _create_test_files(dir_path, m, ext, comment_char='# '):
                 break
 
     test_file = os.path.join(m.path, name)
-    out_file = join(dir_path, 'run_test' + ext)
+    out_file = join(test_dir, 'run_test' + ext)
 
     if os.path.isfile(test_file):
         with open(out_file, 'w') as fo:
@@ -118,8 +111,10 @@ def _create_test_files(dir_path, m, ext, comment_char='# '):
     return (out_file, os.path.isfile(test_file) and os.path.basename(test_file) != 'no-file')
 
 
-def create_py_files(dir_path, m):
-    tf, tf_exists = _create_test_files(dir_path, m, '.py')
+def create_py_files(m, test_dir=None):
+    if not test_dir:
+        test_dir = m.config.test_dir
+    tf, tf_exists = _create_test_files(m, test_dir, '.py')
 
     # Ways in which we can mark imports as none python imports
     # 1. preface package name with r-, lua- or perl-
@@ -142,7 +137,7 @@ def create_py_files(dir_path, m):
         imports = [item for item in imports if (not hasattr(item, 'keys') or
                                                 'lang' in item and item['lang'] == 'python')]
     if imports:
-        with open(tf, 'a+') as fo:
+        with open(tf, 'a') as fo:
             for name in imports:
                 fo.write('print("import: %r")\n' % name)
                 fo.write('import %s\n' % name)
@@ -150,8 +145,10 @@ def create_py_files(dir_path, m):
     return tf if (tf_exists or imports) else False
 
 
-def create_r_files(dir_path, m):
-    tf, tf_exists = _create_test_files(dir_path, m, '.r')
+def create_r_files(m, test_dir=None):
+    if not test_dir:
+        test_dir = m.config.test_dir
+    tf, tf_exists = _create_test_files(m, test_dir, '.r')
 
     imports = None
     # two ways we can enable R import tests:
@@ -166,7 +163,7 @@ def create_r_files(dir_path, m):
                 imports = import_item['imports']
                 break
     if imports:
-        with open(tf, 'a+') as fo:
+        with open(tf, 'a') as fo:
             for name in imports:
                 fo.write('print("library(%r)")\n' % name)
                 fo.write('library(%s)\n' % name)
@@ -174,8 +171,10 @@ def create_r_files(dir_path, m):
     return tf if (tf_exists or imports) else False
 
 
-def create_pl_files(dir_path, m):
-    tf, tf_exists = _create_test_files(dir_path, m, '.pl')
+def create_pl_files(m, test_dir=None):
+    if not test_dir:
+        test_dir = m.config.test_dir
+    tf, tf_exists = _create_test_files(m, test_dir, '.pl')
     imports = None
     if m.name().startswith('perl-'):
         imports = ensure_list(m.get_value('test/imports', []))
@@ -185,8 +184,8 @@ def create_pl_files(dir_path, m):
                     import_item['lang'] == 'perl'):
                 imports = import_item['imports']
                 break
-    if tf or imports:
-        with open(tf, 'a+') as fo:
+    if tf_exists or imports:
+        with open(tf, 'a') as fo:
             print(r'my $expected_version = "%s";' % m.version().rstrip('0'),
                     file=fo)
             if imports:
@@ -206,8 +205,10 @@ def create_pl_files(dir_path, m):
     return tf if (tf_exists or imports) else False
 
 
-def create_lua_files(dir_path, m):
-    tf, tf_exists = _create_test_files(dir_path, m, '.lua')
+def create_lua_files(m, test_dir=None):
+    if not test_dir:
+        test_dir = m.config.test_dir
+    tf, tf_exists = _create_test_files(m, test_dir, '.lua')
     imports = None
     if m.name().startswith('lua-'):
         imports = ensure_list(m.get_value('test/imports', []))
@@ -223,3 +224,15 @@ def create_lua_files(dir_path, m):
                 print(r'print("require \"%s\"\n");' % name, file=fo)
                 print('require "%s"\n' % name, file=fo)
     return tf if (tf_exists or imports) else False
+
+
+def create_all_test_files(m, test_dir=None):
+    if not test_dir:
+        test_dir = m.config.test_dir
+    files = create_files(m, test_dir)
+    pl_files = create_pl_files(m, test_dir)
+    py_files = create_py_files(m, test_dir)
+    r_files = create_r_files(m, test_dir)
+    lua_files = create_lua_files(m, test_dir)
+    shell_files = create_shell_files(m, test_dir)
+    return files, pl_files, py_files, r_files, lua_files, shell_files
